import { APIError } from "./api_error";

export function objForEach<T>(
  obj: T,
  f: (k: keyof T, v: T[keyof T]) => void,
): void {
  for (const k in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, k)) {
      f(k, obj[k]);
    }
  }
}

export type Loading<T> = T | "loading" | "error";

export function loading_success<T>(o: Loading<T>): o is T {
  if (o === "loading") return false;
  if (o === "error") return false;

  return true;
}

export function ensure_loaded<T>(o: Loading<T>): T {
  if (!loading_success(o)) throw "";

  return o;
}

export function handle_error(error: unknown) {
  if (error instanceof APIError) {
    error.alert();
  } else {
    console.error(error);
  }
}

export function name_door(day: number): string {
  return `Türchen ${day}`;
}

export interface Step {
  label: string;
  icon: string | string[];
}
